pub const OUTPUT_NAME: &str = "winit";

use slog::*;
use std::{cell::RefCell, rc::Rc};

use smithay::{
    backend::{
        renderer::{ImportDma, ImportEgl},
        winit,
    },
    reexports::{calloop::EventLoop, wayland_server::Display},
    wayland::{dmabuf::init_dmabuf_global, output::Mode},
};

mod state;
use crate::state::*;

#[cfg(feature = "winit")]
pub struct WinitData;

#[cfg(feature = "winit")]
impl Backend for WinitData {
    fn seat_name(&self) -> String {
        "winit".into()
    }
}

fn main() {
    let log = slog::Logger::root(
        slog_async::Async::default(slog_term::term_full().fuse()).fuse(),
        o!(),
    );
    let _guard = slog_scope::set_global_logger(log.clone());
    slog_stdlog::init().expect("Could not setup log backend");
    run_winit(log)
}

#[cfg(feature = "winit")]
fn run_winit(log: Logger) {

    let mut event_loop = EventLoop::try_new().unwrap();
    let display = Rc::new(RefCell::new(Display::new()));

    let (renderer, mut input) = match winit::init(log.clone()) {
        Ok(ret) => ret,
        Err(_) => return,
    };

    let renderer = Rc::new(RefCell::new(renderer));

    if renderer
        .borrow_mut()
        .renderer()
        .bind_wl_display(&display.borrow())
        .is_ok()
    {
        info!(log, "EGL hardware-acceleration enabled");
        let dmabuf_formats = renderer
            .borrow_mut()
            .renderer()
            .dmabuf_formats()
            .cloned()
            .collect::<Vec<_>>();
        let renderer = renderer.clone();
        init_dmabuf_global(
            &mut *display.borrow_mut(),
            dmabuf_formats,
            move |buffer, _| {
                renderer
                    .borrow_mut()
                    .renderer()
                    .import_dmabuf(buffer)
                    .is_ok()
            },
            log.clone(),
        );
    }

    let size = renderer.borrow().window_size().physical_size;

    let data = WinitData {};

    let mut state = State::init(display.clone(), event_loop.handle(), data, log.clone());

    let mode = Mode {
        size,
        refresh: 60_000,
    };

    loop {}
}
